==> fred-regal-0.5.0.tar.gz: fred-regal-0.5.0/regal/registrar.py <==
"""Registrar client for registry service."""
from datetime import datetime, timedelta, timezone
from decimal import Decimal
from typing import Dict, List, Optional, Sequence, cast

import fred_api.registry.registrar.service_admin_grpc_pb2_grpc
import fred_api.registry.registrar.service_registrar_grpc_pb2_grpc
import grpc
from fred_api.registry.registrar.registrar_common_types_pb2 import Credit
from fred_api.registry.registrar.registrar_credit_types_pb2 import RegistrarCreditReply, RegistrarCreditRequest
from fred_api.registry.registrar.registrar_info_types_pb2 import RegistrarInfoReply, RegistrarInfoRequest
from fred_api.registry.registrar.registrar_zone_access_types_pb2 import (RegistrarZoneAccessReply,
                                                                         RegistrarZoneAccessRequest)
from fred_api.registry.registrar.service_admin_grpc_pb2 import (AddRegistrarZoneAccessReply,
                                                                AddRegistrarZoneAccessRequest, CreateRegistrarRequest,
                                                                DeleteRegistrarZoneAccessReply,
                                                                DeleteRegistrarZoneAccessRequest,
                                                                UpdateRegistrarZoneAccessReply,
                                                                UpdateRegistrarZoneAccessRequest)
from frgal.aio import AsyncGrpcClient
from google.protobuf.empty_pb2 import Empty
from pydantic import BaseModel

from .common import Address, RegistryDecoder
from .constants import NOOP
from .exceptions import HistoryChangeProhibited, InvalidData, RegistrarDoesNotExist


class Registrar(BaseModel):
    """Registrar info.

    Attributes:
        is_internal: Whether the registrar is internal.
    """

    registrar_handle: str
    name: str
    organization: str = ''
    place: Optional[Address] = None

    telephone: Optional[str] = None
    fax: Optional[str] = None
    emails: Sequence[str] = []
    url: Optional[str] = None
    company_registration_number: Optional[str] = None
    vat_identification_number: Optional[str] = None

    is_system_registrar: bool = False
    is_internal: bool = False

    variable_symbol: str = ''
    payment_memo_regex: str = ''
    is_vat_payer: bool = False


class ZoneAccess(BaseModel):
    """Registrar zone access."""

    zone: str
    valid_from: Optional[datetime] = None
    valid_to: Optional[datetime] = None

    def is_active(self) -> bool:
        """Return whether zone access is currently active.

        If `valid_from` is not set, there is no lower bound.
        If `valid_to` is not set, there is no upper bound.
        """
        now = datetime.now(tz=timezone.utc)
        valid_from = self.valid_from or now
        valid_to = self.valid_to or now + timedelta(seconds=1)

        if valid_from <= now < valid_to:
            return True
        else:
            return False


class RegistrarDecoder(RegistryDecoder):
    """Registrar service decoder."""

    def __init__(self) -> None:
        super().__init__()
        self.set_decoder(Credit, self._decode_credit)
        self.set_exception_decoder(RegistrarInfoReply.Exception.RegistrarDoesNotExist, RegistrarDoesNotExist)
        self.set_exception_decoder(RegistrarCreditReply.Exception.RegistrarDoesNotExist, RegistrarDoesNotExist)
        self.set_exception_decoder(RegistrarZoneAccessReply.Exception.RegistrarDoesNotExist, RegistrarDoesNotExist)

        self.set_exception_decoder(AddRegistrarZoneAccessReply.Exception.RegistrarDoesNotExist, RegistrarDoesNotExist)
        self.set_exception_decoder(AddRegistrarZoneAccessReply.Exception.ZoneDoesNotExist, ValueError)
        self.set_exception_decoder(AddRegistrarZoneAccessReply.Exception.InvalidData, InvalidData)
        self.set_exception_decoder(UpdateRegistrarZoneAccessReply.Exception.RegistrarZoneAccessDoesNotExist, ValueError)
        self.set_exception_decoder(
            UpdateRegistrarZoneAccessReply.Exception.HistoryChangeProhibited, HistoryChangeProhibited,
        )
        self.set_exception_decoder(UpdateRegistrarZoneAccessReply.Exception.InvalidData, InvalidData)
        self.set_exception_decoder(DeleteRegistrarZoneAccessReply.Exception.RegistrarZoneAccessDoesNotExist, ValueError)
        self.set_exception_decoder(
            DeleteRegistrarZoneAccessReply.Exception.HistoryChangeProhibited, HistoryChangeProhibited,
        )
        self.set_exception_decoder(DeleteRegistrarZoneAccessReply.Exception.InvalidData, InvalidData)

    def _decode_credit(self, value: Credit) -> Optional[Decimal]:
        """Decode registrar credit."""
        if not value.decimal_value:
            return None
        return Decimal(value.decimal_value)


class RegistrarClient(AsyncGrpcClient):
    """Registrar service gRPC client."""

    service = 'Registrar'

    def __init__(self, netloc: str, credentials: grpc.ChannelCredentials = None) -> None:
        """Initialize Registrar client instance.

        Args:
            netloc: Netloc of registrar service server.
            credentials: Optional credentials.
        """
        super().__init__(
            netloc,
            decoder=RegistrarDecoder(),
            grpc_modules=[fred_api.registry.registrar.service_registrar_grpc_pb2_grpc],
            credentials=credentials,
        )

    async def get_registrar_handles(self) -> List[str]:
        """Get list of all registrar handles."""
        result = await self.call(self.service, 'get_registrar_handles', Empty())
        return cast(List[str], result)

    async def get_registrar_info(self, registrar_handle: str) -> Registrar:
        """Get registrar info by handle."""
        request = RegistrarInfoRequest()
        request.registrar_handle.value = registrar_handle
        result = await self.call(self.service, 'get_registrar_info', request)
        return Registrar(**result)

    async def get_registrar_credit(self, registrar_handle: str) -> Dict[str, Decimal]:
        """Get registrar credit by handle.

        Args:
            registrar_handle: Registrar handle.

        Returns:
            Dictionary with (zone, credit in zone) pairs.
        """
        request = RegistrarCreditRequest()
        request.registrar_handle.value = registrar_handle
        result = await self.call(self.service, 'get_registrar_credit', request)
        return cast(Dict[str, Decimal], result['credit_by_zone'])

    async def get_registrar_zone_access(self, registrar_handle: str) -> Dict[str, bool]:
        """Get registrar zone access.

        Args:
            registrar_handle: Registrar handle.

        Returns:
            Dictionary with (zone, can access zone) pairs.
        """
        request = RegistrarZoneAccessRequest()
        request.registrar_handle.value = registrar_handle
        result = await self.call(self.service, 'get_registrar_zone_access', request)
        return {zone: access['has_access_now'] for zone, access in result['access_by_zone'].items()}

    async def get_registrar_zone_access_history(self, registrar_handle: str) -> Dict[str, List[ZoneAccess]]:
        """Get registrar zone access history.

        Args:
            registrar_handle: Registrar handle.

        Returns:
            Dictionary with zones as keys and list of zone accesses as values.
        """
        request = RegistrarZoneAccessRequest()
        request.registrar_handle.value = registrar_handle
        result = await self.call(self.service, 'get_registrar_zone_access', request)
        zone_access = result['access_by_zone']
        for zone, access in zone_access.items():
            zone_access[zone] = [
                ZoneAccess(zone=zone, valid_from=acc['valid_from'], valid_to=acc['valid_to'])
                for acc in access['history']
            ]
        return cast(Dict[str, List[ZoneAccess]], zone_access)


class RegistrarAdminClient(AsyncGrpcClient):
    """Registrar Admin service gRPC client."""

    decoder_cls = RegistrarDecoder
    grpc_modules = (fred_api.registry.registrar.service_admin_grpc_pb2_grpc,)
    service = "Admin"

    async def create_registrar(self, registrar: Registrar) -> None:
        """Create a registrar.

        Args:
            registrar: The registrar object.
        """
        request = CreateRegistrarRequest()
        request.registrar_handle.value = registrar.registrar_handle
        request.name = registrar.name
        request.organization = registrar.organization
        if registrar.place is not None:
            for street in registrar.place.street:
                request.place.street.append(street)
            request.place.city = registrar.place.city
            request.place.state_or_province = registrar.place.state_or_province
            request.place.postal_code.value = registrar.place.postal_code
            request.place.country_code.value = registrar.place.country_code
        if registrar.telephone is not None:
            request.telephone.value = registrar.telephone
        if registrar.fax is not None:
            request.fax.value = registrar.fax
        for email in registrar.emails:
            request.emails.add(value=email)
        if registrar.url is not None:
            request.url.value = registrar.url
        request.is_system_registrar = registrar.is_system_registrar
        if registrar.company_registration_number is not None:
            request.company_registration_number.value = registrar.company_registration_number
        if registrar.vat_identification_number is not None:
            request.vat_identification_number.value = registrar.vat_identification_number
        request.variable_symbol = registrar.variable_symbol
        request.payment_memo_regex = registrar.payment_memo_regex
        request.is_vat_payer = registrar.is_vat_payer
        request.is_internal = registrar.is_internal
        await self.call(self.service, 'create_registrar', request)

    async def add_zone_access(self, registrar_handle: str, access: ZoneAccess) -> ZoneAccess:
        """Add registrar zone access.

        Args:
            registrar_handle: Registrar handle.
            access: Zone access to add.

        Returns:
            Added zone access.
        """
        request = AddRegistrarZoneAccessRequest()
        request.registrar_handle.value = registrar_handle
        request.zone = access.zone
        if access.valid_from:
            request.valid_from.FromDatetime(access.valid_from)
        if access.valid_to:
            request.valid_to.FromDatetime(access.valid_to)
        result = await self.call(self.service, "add_registrar_zone_access", request)
        return ZoneAccess(**result)

    async def update_zone_access(
        self,
        registrar_handle: str,
        access: ZoneAccess,
        *,
        valid_from: Optional[datetime] = NOOP,
        valid_to: Optional[datetime] = NOOP,
    ) -> ZoneAccess:
        """Update registrar zone access.

        Args:
            registrar_handle: Registrar handle.
            access: Zone access to update. `valid_from` has to be defined.
            valid_from: Future `valid_from` that replaces current value. If None, current date will be used.
            valid_to: New `valid_to` that replaces current value. If None, validity is not limited.

        Returns:
            Updated zone access.
        """
        request = UpdateRegistrarZoneAccessRequest()
        request.registrar_handle.value = registrar_handle
        request.zone = access.zone
        request.valid_from.FromDatetime(access.valid_from)

        if isinstance(valid_from, datetime):
            request.set_valid_from.FromDatetime(valid_from)
        elif valid_from is None:
            request.set_valid_from.SetInParent()

        if isinstance(valid_to, datetime):
            request.set_valid_to.FromDatetime(valid_to)
        elif valid_to is None:
            request.set_valid_to.SetInParent()

        result = await self.call(self.service, "update_registrar_zone_access", request)
        return ZoneAccess(**result)

    async def delete_zone_access(self, registrar_handle: str, access: ZoneAccess) -> None:
        """Delete registrar zone access.

        Args:
            registrar_handle: Registrar handle.
            access: Zone access to update. `valid_from` has to be defined.
        """
        request = DeleteRegistrarZoneAccessRequest()
        request.registrar_handle.value = registrar_handle
        request.zone = access.zone
        request.valid_from.FromDatetime(access.valid_from)
        await self.call(self.service, "delete_registrar_zone_access", request)

==> fred-regal-0.5.1.tar.gz: fred-regal-0.5.1/regal/registrar.py <==
"""Registrar client for registry service."""
from datetime import datetime, timedelta, timezone
from decimal import Decimal
from typing import Dict, List, Optional, Sequence, cast

import fred_api.registry.registrar.service_admin_grpc_pb2_grpc
import fred_api.registry.registrar.service_registrar_grpc_pb2_grpc
import grpc
from fred_api.registry.registrar.registrar_common_types_pb2 import Credit
from fred_api.registry.registrar.registrar_credit_types_pb2 import RegistrarCreditReply, RegistrarCreditRequest
from fred_api.registry.registrar.registrar_info_types_pb2 import RegistrarInfoReply, RegistrarInfoRequest
from fred_api.registry.registrar.registrar_zone_access_types_pb2 import (RegistrarZoneAccessReply,
                                                                         RegistrarZoneAccessRequest)
from fred_api.registry.registrar.service_admin_grpc_pb2 import (AddRegistrarZoneAccessReply,
                                                                AddRegistrarZoneAccessRequest, CreateRegistrarRequest,
                                                                DeleteRegistrarZoneAccessReply,
                                                                DeleteRegistrarZoneAccessRequest, UpdateRegistrarReply,
                                                                UpdateRegistrarRequest, UpdateRegistrarZoneAccessReply,
                                                                UpdateRegistrarZoneAccessRequest)
from frgal.aio import AsyncGrpcClient
from google.protobuf.empty_pb2 import Empty
from pydantic import BaseModel

from .common import Address, RegistryDecoder
from .constants import NOOP
from .exceptions import HistoryChangeProhibited, InvalidData, RegistrarDoesNotExist


class Registrar(BaseModel):
    """Registrar info.

    Attributes:
        is_internal: Whether the registrar is internal.
    """

    registrar_handle: str
    name: str
    organization: str = ''
    place: Optional[Address] = None

    telephone: Optional[str] = None
    fax: Optional[str] = None
    emails: Sequence[str] = []
    url: Optional[str] = None
    company_registration_number: Optional[str] = None
    vat_identification_number: Optional[str] = None

    is_system_registrar: bool = False
    is_internal: bool = False

    variable_symbol: str = ''
    payment_memo_regex: str = ''
    is_vat_payer: bool = False


class ZoneAccess(BaseModel):
    """Registrar zone access."""

    zone: str
    valid_from: Optional[datetime] = None
    valid_to: Optional[datetime] = None

    def is_active(self) -> bool:
        """Return whether zone access is currently active.

        If `valid_from` is not set, there is no lower bound.
        If `valid_to` is not set, there is no upper bound.
        """
        now = datetime.now(tz=timezone.utc)
        valid_from = self.valid_from or now
        valid_to = self.valid_to or now + timedelta(seconds=1)

        if valid_from <= now < valid_to:
            return True
        else:
            return False


class RegistrarDecoder(RegistryDecoder):
    """Registrar service decoder."""

    def __init__(self) -> None:
        super().__init__()
        self.set_decoder(Credit, self._decode_credit)
        self.set_exception_decoder(RegistrarInfoReply.Exception.RegistrarDoesNotExist, RegistrarDoesNotExist)
        self.set_exception_decoder(RegistrarCreditReply.Exception.RegistrarDoesNotExist, RegistrarDoesNotExist)
        self.set_exception_decoder(RegistrarZoneAccessReply.Exception.RegistrarDoesNotExist, RegistrarDoesNotExist)

        self.set_exception_decoder(AddRegistrarZoneAccessReply.Exception.RegistrarDoesNotExist, RegistrarDoesNotExist)
        self.set_exception_decoder(AddRegistrarZoneAccessReply.Exception.ZoneDoesNotExist, ValueError)
        self.set_exception_decoder(AddRegistrarZoneAccessReply.Exception.InvalidData, InvalidData)
        self.set_exception_decoder(UpdateRegistrarZoneAccessReply.Exception.RegistrarZoneAccessDoesNotExist, ValueError)
        self.set_exception_decoder(
            UpdateRegistrarZoneAccessReply.Exception.HistoryChangeProhibited, HistoryChangeProhibited,
        )
        self.set_exception_decoder(UpdateRegistrarZoneAccessReply.Exception.InvalidData, InvalidData)
        self.set_exception_decoder(DeleteRegistrarZoneAccessReply.Exception.RegistrarZoneAccessDoesNotExist, ValueError)
        self.set_exception_decoder(
            DeleteRegistrarZoneAccessReply.Exception.HistoryChangeProhibited, HistoryChangeProhibited,
        )
        self.set_exception_decoder(DeleteRegistrarZoneAccessReply.Exception.InvalidData, InvalidData)
        self.set_exception_decoder(UpdateRegistrarReply.Exception.RegistrarDoesNotExist, RegistrarDoesNotExist)
        self.set_exception_decoder(UpdateRegistrarReply.Exception.InvalidData, InvalidData)

    def _decode_credit(self, value: Credit) -> Optional[Decimal]:
        """Decode registrar credit."""
        if not value.decimal_value:
            return None
        return Decimal(value.decimal_value)


class RegistrarClient(AsyncGrpcClient):
    """Registrar service gRPC client."""

    service = 'Registrar'

    def __init__(self, netloc: str, credentials: grpc.ChannelCredentials = None) -> None:
        """Initialize Registrar client instance.

        Args:
            netloc: Netloc of registrar service server.
            credentials: Optional credentials.
        """
        super().__init__(
            netloc,
            decoder=RegistrarDecoder(),
            grpc_modules=[fred_api.registry.registrar.service_registrar_grpc_pb2_grpc],
            credentials=credentials,
        )

    async def get_registrar_handles(self) -> List[str]:
        """Get list of all registrar handles."""
        result = await self.call(self.service, 'get_registrar_handles', Empty())
        return cast(List[str], result)

    async def get_registrar_info(self, registrar_handle: str) -> Registrar:
        """Get registrar info by handle."""
        request = RegistrarInfoRequest()
        request.registrar_handle.value = registrar_handle
        result = await self.call(self.service, 'get_registrar_info', request)
        return Registrar(**result)

    async def get_registrar_credit(self, registrar_handle: str) -> Dict[str, Decimal]:
        """Get registrar credit by handle.

        Args:
            registrar_handle: Registrar handle.

        Returns:
            Dictionary with (zone, credit in zone) pairs.
        """
        request = RegistrarCreditRequest()
        request.registrar_handle.value = registrar_handle
        result = await self.call(self.service, 'get_registrar_credit', request)
        return cast(Dict[str, Decimal], result['credit_by_zone'])

    async def get_registrar_zone_access(self, registrar_handle: str) -> Dict[str, bool]:
        """Get registrar zone access.

        Args:
            registrar_handle: Registrar handle.

        Returns:
            Dictionary with (zone, can access zone) pairs.
        """
        request = RegistrarZoneAccessRequest()
        request.registrar_handle.value = registrar_handle
        result = await self.call(self.service, 'get_registrar_zone_access', request)
        return {zone: access['has_access_now'] for zone, access in result['access_by_zone'].items()}

    async def get_registrar_zone_access_history(self, registrar_handle: str) -> Dict[str, List[ZoneAccess]]:
        """Get registrar zone access history.

        Args:
            registrar_handle: Registrar handle.

        Returns:
            Dictionary with zones as keys and list of zone accesses as values.
        """
        request = RegistrarZoneAccessRequest()
        request.registrar_handle.value = registrar_handle
        result = await self.call(self.service, 'get_registrar_zone_access', request)
        zone_access = result['access_by_zone']
        for zone, access in zone_access.items():
            zone_access[zone] = [
                ZoneAccess(zone=zone, valid_from=acc['valid_from'], valid_to=acc['valid_to'])
                for acc in access['history']
            ]
        return cast(Dict[str, List[ZoneAccess]], zone_access)


class RegistrarAdminClient(AsyncGrpcClient):
    """Registrar Admin service gRPC client."""

    decoder_cls = RegistrarDecoder
    grpc_modules = (fred_api.registry.registrar.service_admin_grpc_pb2_grpc,)
    service = "Admin"

    async def create_registrar(self, registrar: Registrar) -> None:
        """Create a registrar.

        Args:
            registrar: The registrar object.
        """
        request = CreateRegistrarRequest()
        request.registrar_handle.value = registrar.registrar_handle
        request.name = registrar.name
        request.organization = registrar.organization
        if registrar.place is not None:
            for street in registrar.place.street:
                request.place.street.append(street)
            request.place.city = registrar.place.city
            request.place.state_or_province = registrar.place.state_or_province
            request.place.postal_code.value = registrar.place.postal_code
            request.place.country_code.value = registrar.place.country_code
        if registrar.telephone is not None:
            request.telephone.value = registrar.telephone
        if registrar.fax is not None:
            request.fax.value = registrar.fax
        for email in registrar.emails:
            request.emails.add(value=email)
        if registrar.url is not None:
            request.url.value = registrar.url
        request.is_system_registrar = registrar.is_system_registrar
        if registrar.company_registration_number is not None:
            request.company_registration_number.value = registrar.company_registration_number
        if registrar.vat_identification_number is not None:
            request.vat_identification_number.value = registrar.vat_identification_number
        request.variable_symbol = registrar.variable_symbol
        request.payment_memo_regex = registrar.payment_memo_regex
        request.is_vat_payer = registrar.is_vat_payer
        request.is_internal = registrar.is_internal
        await self.call(self.service, 'create_registrar', request)

    async def update_registrar(  # noqa: C901
        self,
        registrar_handle: str,
        *,
        name: str = None,
        organization: str = None,
        place: Address = None,
        telephone: str = None,
        fax: str = None,
        emails: Sequence[str] = None,
        url: str = None,
        is_system_registrar: bool = None,
        company_registration_number: str = None,
        vat_identification_number: str = None,
        variable_symbol: str = None,
        payment_memo_regex: str = None,
        is_vat_payer: bool = None,
        is_internal: bool = None,
    ) -> None:
        """Update a registrar.

        Args:
            registrar_handle: Registrar handle.
            name: Set new name.
            organization: Set new organization.
            place: Set new address.
            telephone: Set new phone number.
            fax: Set new fax.
            emails: Set new list of emails.
            url: Set new url.
            is_system_registrar: Set whether registrar is system registrar.
            company_registration_number: Set new company registration number.
            vat_identification_number: Set new VAT identification number.
            variable_symbol: Set new variable symbol.
            payment_memo_regex: Set new payment memo regex.
            is_vat_payer: Set whether registrar is VAT payer.
            is_internal: Set whether registrar is internal.
        """
        request = UpdateRegistrarRequest()
        request.registrar_handle.value = registrar_handle
        if name is not None:
            request.set_name = name
        if organization is not None:
            request.set_organization = organization
        if place is not None:
            request.set_place.street[:] = place.street
            request.set_place.city = place.city
            request.set_place.state_or_province = place.state_or_province
            if place.postal_code is not None:
                request.set_place.postal_code.value = place.postal_code
            if place.country_code is not None:
                request.set_place.country_code.value = place.country_code
        if telephone is not None:
            request.set_telephone.value = telephone
        if fax is not None:
            request.set_fax.value = fax
        if emails is not None:
            for email in emails:
                request.set_emails.values.add(value=email)
        if url is not None:
            request.set_url.value = url
        if is_system_registrar is not None:
            request.set_is_system_registrar = is_system_registrar
        if company_registration_number is not None:
            request.set_company_registration_number.value = company_registration_number
        if vat_identification_number is not None:
            request.set_vat_identification_number.value = vat_identification_number
        if variable_symbol is not None:
            request.set_variable_symbol = variable_symbol
        if payment_memo_regex is not None:
            request.set_payment_memo_regex = payment_memo_regex
        if is_vat_payer is not None:
            request.set_is_vat_payer = is_vat_payer
        if is_internal is not None:
            request.set_is_internal = is_internal
        await self.call(self.service, "update_registrar", request)

    async def add_zone_access(self, registrar_handle: str, access: ZoneAccess) -> ZoneAccess:
        """Add registrar zone access.

        Args:
            registrar_handle: Registrar handle.
            access: Zone access to add.

        Returns:
            Added zone access.
        """
        request = AddRegistrarZoneAccessRequest()
        request.registrar_handle.value = registrar_handle
        request.zone = access.zone
        if access.valid_from:
            request.valid_from.FromDatetime(access.valid_from)
        if access.valid_to:
            request.valid_to.FromDatetime(access.valid_to)
        result = await self.call(self.service, "add_registrar_zone_access", request)
        return ZoneAccess(**result)

    async def update_zone_access(
        self,
        registrar_handle: str,
        access: ZoneAccess,
        *,
        valid_from: Optional[datetime] = NOOP,
        valid_to: Optional[datetime] = NOOP,
    ) -> ZoneAccess:
        """Update registrar zone access.

        Args:
            registrar_handle: Registrar handle.
            access: Zone access to update. `valid_from` has to be defined.
            valid_from: Future `valid_from` that replaces current value. If None, current date will be used.
            valid_to: New `valid_to` that replaces current value. If None, validity is not limited.

        Returns:
            Updated zone access.
        """
        request = UpdateRegistrarZoneAccessRequest()
        request.registrar_handle.value = registrar_handle
        request.zone = access.zone
        request.valid_from.FromDatetime(access.valid_from)

        if isinstance(valid_from, datetime):
            request.set_valid_from.FromDatetime(valid_from)
        elif valid_from is None:
            request.set_valid_from.SetInParent()

        if isinstance(valid_to, datetime):
            request.set_valid_to.FromDatetime(valid_to)
        elif valid_to is None:
            request.set_valid_to.SetInParent()

        result = await self.call(self.service, "update_registrar_zone_access", request)
        return ZoneAccess(**result)

    async def delete_zone_access(self, registrar_handle: str, access: ZoneAccess) -> None:
        """Delete registrar zone access.

        Args:
            registrar_handle: Registrar handle.
            access: Zone access to update. `valid_from` has to be defined.
        """
        request = DeleteRegistrarZoneAccessRequest()
        request.registrar_handle.value = registrar_handle
        request.zone = access.zone
        request.valid_from.FromDatetime(access.valid_from)
        await self.call(self.service, "delete_registrar_zone_access", request)
